
function dec2bin(val) {
	if (!parseInt(val))
		return false;
	if (val > 65535)
		return false;
	return String("0000000000000000"+parseInt(val).toString(2)).slice(-16);
}

function getMask(bits) {
	switch(bits) {
		case 0:  return "ffff"; break;
		case 1:  return "fffe"; break;
		case 2:  return "fffc"; break;
		case 3:  return "fff8"; break;
		case 4:  return "fff0"; break;
		case 5:  return "ffe0"; break;
		case 6:  return "ffc0"; break;
		case 7:  return "ff80"; break;
		case 8:  return "ff00"; break;
		case 9:  return "fe00"; break;
		case 10: return "fc00"; break;
		case 11: return "f800"; break;
		case 12: return "f000"; break;
		case 13: return "e000"; break;
		case 14: return "c000"; break;
		case 15: return "8000"; break;
		case 16: return "0000"; break;
	}
	return false;
}

function binLower(bin, bits) {
	var outBin = bin.substr(0, 16-bits);
	for (var i = 0; i < bits; i++) {
		outBin += "0";
	}
	return parseInt(outBin,2);
}

function binHigher(bin, bits) {
	var outBin = bin.substr(0, 16-bits);
	for (var i = 0; i < bits; i++) {
		outBin += "1";
	}
	return parseInt(outBin,2);
}

function testMasks(bin, start, end) {
	for (var mask = 0; mask <=16; mask++) {
		var maskStart = binLower(bin, mask);
		var maskEnd = binHigher(bin, mask);
		if (maskStart < start || maskEnd > end) {
			output.append("<em style='color: red;'>Failed mask: "+getMask(mask)+". Resulting range: "+maskStart+"-"+maskEnd+"!<br>");
			return mask-1;
		}
	}
	return mask;
}

var container = [];

function masksStart() {
	container = [];
	output = $("#u32-output");
	output.text("");

	var range = [];
	range.start = parseInt($("#range_start").val());
	range.end = parseInt($("#range_end").val());

	if (isNaN(range.end) || isNaN(range.start)) {
		output.text("Ports must be numbers.");
		return;
	}
	if (range.end < range.start) {	
		output.text("First port value must be lower than the second.");
		return;
	}

	if (range.start <= 0 || range.start > 65535) {
		output.text("Valid range: 1-65535");
	}
	if (range.end <= 0 || range.end > 65535) {
		output.text("Valid range: 1-65535");
	}
	
	var processing = true;
	output.append("<br><b>Trying out ranges by expanding the mask:<b><br>");
	while(processing) {
		
		var bin = [];
		bin.start = dec2bin(range.start);
		bin.lastOne = bin.start.lastIndexOf("1")+1;
		bin.bitsToChange = 16 - bin.lastOne;

		var mask = testMasks(bin.start, range.start, range.end);

		var maskStart = binLower(bin.start, mask);
		var maskEnd = binHigher(bin.start, mask);

		output.append("<b style='color:green;'>Best valid mask: "+getMask(mask)+". Resulting range: "+maskStart+"-"+maskEnd+"...<br><br>");
		var rangeObj = [];
		rangeObj.port = range.start;
		rangeObj.mask = getMask(mask);
		container.push(rangeObj);

		range.start = maskEnd + 1;
		if (range.start > range.end) processing = false;
	}
	
	output.append("<p>A total of "+container.length+" filters will be necessary to cover the port range "+$("#range_start").val()+"-"+$("#range_end").val()+"...</p>Here some example filters:<br>");
	for (var i = 0; i < container.length; i++) {
		output.append("tc filter add (...) u32 match ip [s-d]port "+container[i].port+" 0x"+container[i].mask+" flowid (...)<br>");
	}
}

