#!/bin/bash
#
# Script permettant la réactualisation du débit réel de la connexion internet
# Script exécuté par un cron

#Chargement des dicos
. ParseDico

LOG_DIJON="/var/log/amon_dijon.log"
DAY=`date +"%d-%B-%Y-%H:%M"`
NOM_SCRIPT=`echo ${0##*/}`

# Vérifie si le script est déjà en cours d'exécution
if pidof -x $(basename $0) > /dev/null; then
  for p in $(pidof -x $(basename $0)); do
    if [ $p -ne $$ ]; then
      echo "$NOM_SCRIPT est déjà en cours d'exécution: Fin du script"
      echo "$DAY --- $NOM_SCRIPT --- Script déjà en cours d'exécution ---" >> $LOG_DIJON
      exit 1
    fi
  done
fi

DOWNRATE=$(</usr/src/qos_downrate)

# Test la présence du script speedtest-cli
if [ ! -s /usr/src/speedtest-cli ]
then echo "Script speedtest-cli absent"
     echo "$DAY --- $NOM_SCRIPT --- Script speedtest-cli absent - Fin script QoS ---" >> $LOG_DIJON
     echo "QoS AMON $nom_machine : Script speedtest-cli absent le $DAY"|mutt -s "[QoS Amon] $nom_machine : speedtest-cli absent" "$qos_mail_dest" -e 'my_hdr From: '$nom_machine' <'$qos_mail_dest'>'
     exit 1
fi

# Test accès serveur speedtest
/usr/src/speedtest-cli --list > /dev/null  2>&1
if [ $? != 0 ]
then echo "Serveur speedtest.net non accessible"
     echo "$DAY --- $NOM_SCRIPT --- Serveur speedtest.net inaccessible - Fin script QoS ---" >> $LOG_DIJON
     echo "QoS AMON $nom_machine : Serveur speedtest.net inaccessible le $DAY"|mutt -s "[QoS Amon] $nom_machine : speedtest.net inaccessible" "$qos_mail_dest" -e 'my_hdr From: '$nom_machine' <'$qos_mail_dest'>'
     exit 1
fi

# Désactivation de la QoS pour le test
/sbin/tc qdisc del dev eth0 root
/sbin/tc qdisc del dev eth0 ingress
/sbin/tc qdisc del dev ifb0 root
/sbin/tc qdisc del dev ifb0 ingress
/sbin/rmmod ifb

# Désactivation des interfaces eth1, eth2 et eth3 pour ne pas perturber le test de débit
ifdown eth1 > /dev/null  2>&1; ifdown eth2 > /dev/null  2>&1; ifdown eth3 > /dev/null  2>&1

# Démarrage du test de débit
echo "Test en cours"
CRONDOWNRATE=$(/usr/src/speedtest-cli --bytes --simple | awk '/Download/ {print $2}')

# Activation des interfaces eth1, eth2 et eth3
ifup eth1 > /dev/null  2>&1; ifup eth2 > /dev/null  2>&1; ifup eth3 > /dev/null  2>&1

# Tests sur le résultat
if [ -z $CRONDOWNRATE ]
    then echo "Erreur de calcul du débit - Fin du script QoS"
         echo "$DAY --- $NOM_SCRIPT --- Erreur de calcul du débit ---" >> $LOG_DIJON
         echo "QoS AMON $nom_machine : Erreur de calcul de débit le $DAY"|mutt -s "[QoS Amon] $nom_machine : Erreur calcul débit" "$qos_mail_dest" -e 'my_hdr From: '$nom_machine' <'$qos_mail_dest'>'
         exit 1
    else if [ "$CRONDOWNRATE" -lt "$qos_debit_alerte" ]
            then echo "$DAY --- $NOM_SCRIPT --- Débit calculé ($CRONDOWNRATE Ko/s) inférieur au seuil ($qos_debit_alerte Ko/s)---" >> $LOG_DIJON
                 echo "QoS AMON $nom_machine : Débit calculé ($CRONDOWNRATE Ko/s) inférieur au seuil ($qos_debit_alerte Ko/s) le $DAY"|mutt -s "[QoS Amon] $nom_machine : Alerte faible débit" "$qos_mail_dest" -e 'my_hdr From: '$nom_machine' <'$qos_mail_dest'>'
         fi
         if [ $(($CRONDOWNRATE*100/$DOWNRATE)) -lt 70 ] || [ $(($CRONDOWNRATE*100/$DOWNRATE)) -gt 130 ]
	    then echo "Débit calculé ($CRONDOWNRATE Ko/s) a plus de 30% d'écart avec le débit précédent ($DOWNRATE Ko/s)"
                 echo "$DAY --- $NOM_SCRIPT --- Débit calculé ($CRONDOWNRATE Ko/s) a plus de 30% d'écart avec le débit précédent ($DOWNRATE Ko/s)---" >> $LOG_DIJON
                 echo "QoS AMON $nom_machine : Variation importante du débit calculé ($CRONDOWNRATE Ko/s) le $DAY par rapport au débit précédent ($DOWNRATE Ko/s)"|mutt -s "[QoS Amon] $nom_machine : Variation importante calcul débit" "$qos_mail_dest" -e 'my_hdr From: '$nom_machine' <'$qos_mail_dest'>'	
		 fi
		 echo $CRONDOWNRATE > /usr/src/qos_downrate
         echo "$DAY --- $NOM_SCRIPT --- Débit calculé à $CRONDOWNRATE Ko/s ---" >> $LOG_DIJON
fi
